import javax.swing.Action
import org.slf4j.LoggerFactory
import com.tagtraum.beatunes.action.standard.LibraryBatchAction
import com.tagtraum.beatunes.action.standard.LibraryBatchAction.EachSongProcessor
import com.tagtraum.beatunes.library.Song
import com.tagtraum.beatunes.library.MediaLibrary
import com.tagtraum.audiokern.AudioSong
import com.tagtraum.tunes.TunesUtilities;

class ResetRatings extends LibraryBatchAction {

    // Inner class that implements the
    // com.tagtraum.beatunes.action.standard.LibraryBatchAction.EachSongProcessor
    // interface. Its process method is called for each song.
    class Adjuster implements EachSongProcessor {

        static log = LoggerFactory.getLogger("ResetRatings.groovy")

        // Called once, before processing starts.
        def void startProcessing(int count) {
            log.info "We are expecting to process ${count} tracks. Let's start!"
        }

        // Called for each song.
        def void process(AudioSong song, int index) {
            // print to beaTunes log file
            log.info "Song: ${song.getName()}"
            try {
                if (!song.isRatingComputed()) {
                    log.info "Resetting rating to ${song.getRating()}"
                    song.setRating(song.getRating())
                    TunesUtilities.invokeAndWait({}, 1, java.util.concurrent.TimeUnit.MINUTES)
                }
            } catch (Exception e) {
                // something went sideways, let's log it
                log.error("Something went sideways: ${e}", e)
            }
        }

        // Called once all songs were processed.
        def void finishProcessing() {
            log.info "Done!"
        }

        // Message to be shown in progress dialog.
        def String getProgressDialogMessage(AudioSong song) {
            "<html>Resetting rating for <i>${song.getName()}</i> ...</html>"
        }

        // Title for progress dialog.
        def String getProgressDialogTitle() {
            "Resetting Ratings ..."
        }
    }

    // Unique id
    def String getId() {
        "Groovy.ResetRating"
    }

    // Is called by beaTunes as part of the lifecycle after instantiation.
    // At this point all other plugins are instantiated and registered.
    // We use this to set the menu item's (i.e. action's) name.
    def void init() {
        putValue(Action.NAME, "Reset Ratings")
    }

    // We need to ask the user, whether he really wants to do this.
    // How we ask is defined here.
    def String getConfirmationMessage() {
        "Do you really want to reset the ratings of all songs in Music/iTunes?"
    }

    def MediaLibrary getMediaLibrary() {
        return getApplication().getMediaLibrary()
    }

    // Factory method that creates the processor for each song.
    def EachSongProcessor createEachSongProcessor() {
        new Adjuster()
    }
}
